<?php

namespace Drupal\drd\Plugin\Action;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\Action\ActionBase;
use Drupal\Core\Session\AccountInterface;

/**
 * Base class for DRD Action plugins.
 */
abstract class Base extends ActionBase implements BaseInterface {

  /** @var array */
  protected $arguments = array();

  /** @var string|string[] */
  protected $output;

  /** @var \Symfony\Component\Console\Style\SymfonyStyle */
  protected $io;

  /**
   * @inheritDoc
   */
  public function restrictAccess() {
    return TRUE;
  }

  /**
   * @inheritDoc
   */
  final public function setIo($io) {
    $this->io = $io;
  }

  /**
   * @return string|bool
   */
  protected function getFollowUpAction() {
    return FALSE;
  }

  /**
   * @param string $id
   *
   * @return Base|bool
   */
  final public static function instance($id) {
    $type = \Drupal::service('plugin.manager.action');
    /** @var BaseEntityRemote $action */
    $action = $type->createInstance($id);
    if (empty($action) || !($action instanceof BaseInterface)) {
      return FALSE;
    }
    return $action;
  }

  /**
   * @inheritDoc
   */
  public function access($object, AccountInterface $account = NULL, $return_as_object = FALSE) {
    if (PHP_SAPI == 'cli' || @ignore_user_abort()) {
      // We are either running via CLI (Drush or Console) or it's a cron run.
      return $return_as_object ? AccessResult::allowed() : TRUE;
    }
    if (!$account) {
      $account = \Drupal::currentUser();
    }
    if ($account->hasPermission($this->getPluginId())) {
      return $return_as_object ? AccessResult::allowed() : TRUE;
    }
    return $return_as_object ? AccessResult::forbidden() : FALSE;
  }

  /**
   * @inheritDoc
   */
  public function execute() {
    // TODO: Implement execute() method.
  }

  /**
   * @inheritDoc
   */
  final public function setActionArgument($key, $value) {
    $this->arguments[$key] = $value;
  }

  /**
   * @inheritDoc
   */
  final public function setArguments(array $arguments) {
    $this->arguments = $arguments;
  }

  /**
   * @inheritDoc
   */
  final public function getArguments() {
    return $this->arguments;
  }

  /**
   * @inheritDoc
   */
  final public function setOutput($output) {
    $this->output[] = $output;
  }

  /**
   * @inheritDoc
   */
  final public function getOutput() {
    return empty($this->output) ? FALSE : $this->output;
  }

}
