<?php

namespace Drupal\drd\Plugin\Action;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\Session\AccountInterface;

/**
 * Base class for DRD Action plugins.
 */
abstract class BaseEntity extends Base implements BaseEntityInterface {

  /** @var \Drupal\drd\Entity\BaseInterface */
  protected $drdEntity;

  /**
   * {@inheritdoc}
   */
  public function access($domain, AccountInterface $account = NULL, $return_as_object = FALSE) {
    if (!isset($domain)) {
      // This is a general check if we have access to the action itself.
      return parent::access($domain, $account, $return_as_object);
    }

    $this->drdEntity = $domain;
    if (parent::access($domain, $account)) {
      return $this->drdEntity->access('edit', $account, $return_as_object);
    }
    return $return_as_object ? AccessResult::forbidden() : FALSE;
  }

  /**
   * @param \Drupal\Core\Entity\EntityInterface $entity
   * @param string $severity
   * @param string $message
   * @param array $arguments
   */
  protected function log($entity, $severity, $message, $arguments = []) {
    if (!method_exists(\Drupal::logger('drd'), $severity)) {
      $severity = 'emergency';
    }
    $arguments['@plugin_id'] = $this->pluginId;
    $arguments['@entity_id'] = $entity->id();
    $arguments['@entity_name'] = $entity->label();
    $arguments['@entity_type'] = $entity->getEntityTypeId();
    $arguments['link'] = $entity->toUrl()->toString(TRUE)->getGeneratedUrl();
    $message = '@plugin_id [@entity_type/@entity_id]: ' . $message . '<br>@entity_name';
    if (!isset($this->io)) {
      \Drupal::logger('drd')->{$severity}($message, $arguments);
    }
    else {
      $this->io->note($this->t($message, $arguments)->render());
    }
  }

}
